'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const wikidot_ajax_1 = require("./wikidot-ajax");
const XMLRPC = require("xmlrpc");
const Bluebird = require("bluebird");
const axios_1 = require("axios");
const querystring_1 = require("querystring");
const CONCURRENCY = 4;
class WikidotKit {
    constructor(token) {
        if (!token) {
            throw new Error('token is required');
        }
        this._xmlrpc = XMLRPC.createSecureClient({
            host: 'www.wikidot.com',
            port: 443,
            path: '/xml-rpc-api.php',
            basic_auth: {
                user: 'WikidotKit',
                pass: token
            }
        });
    }
    call(wiki, method, args = {}) {
        const callArgs = Object.assign({}, args, { site: wiki });
        return new Bluebird((resolve, reject) => {
            this._xmlrpc.methodCall(method, [callArgs], (error, result) => {
                if (error) {
                    reject(error);
                }
                else {
                    resolve(result);
                }
            });
        });
    }
    fetchPagesList(wiki) {
        return this.call(wiki, 'pages.select');
    }
    fetchPage(wiki, name) {
        return this.call(wiki, 'pages.get_one', { site: wiki, page: name });
    }
    fetchMembersList(wikiUrl) {
        const query = wikidot_ajax_1.default(wikiUrl);
        return query({
            moduleName: 'membership/MembersListModule'
        })
            .then(($) => {
            const totalPages = parseInt($('.pager .target:nth-last-child(2)').text(), 10);
            const pages = [];
            for (let i = 1; i <= totalPages; i++) {
                pages.push(i);
            }
            return pages;
        })
            .map((pageNumber) => {
            return query({
                moduleName: 'membership/MembersListModule',
                page: pageNumber
            });
        }, { concurrency: CONCURRENCY })
            .map(($) => {
            return Array.from($('.printuser a:last-of-type')).map((elem) => {
                const jqElem = $(elem);
                return {
                    username: jqElem.text(),
                    uid: Number(jqElem.attr('onclick').replace(/.*\((.*?)\).*/, '$1'))
                };
            });
        })
            .reduce((allUsers, pageOfUsers) => allUsers.concat(pageOfUsers), []);
    }
    fetchUserProfile(wikiUrl, uid) {
        const query = wikidot_ajax_1.default(wikiUrl);
        return query({
            moduleName: 'users/UserInfoWinModule',
            user_id: uid
        })
            .then(($) => {
            const username = $('h1').text();
            const about = $('.table tr em').text();
            const date = $('.table tr .odate');
            const userSince = new Date($(date[0]).text());
            const memberSince = new Date($(date[1]).text());
            if (username.length) {
                return { uid, username, about, userSince, memberSince };
            }
            else {
                return { uid, deleted: true };
            }
        });
    }
    async resolvePageId(pageURL) {
        const PAGE_ID_REGEXP = /WIKIREQUEST\.info\.pageId = (\d+);/;
        const page = await axios_1.default({ url: pageURL });
        const match = PAGE_ID_REGEXP.exec(page.data);
        return match && parseInt(match[1], 10) || null;
    }
    async fetchPageVotes(wikiUrl, pageName) {
        const pageUrl = `${wikiUrl}/${pageName}`;
        const pageID = await this.resolvePageId(pageUrl);
        if (!pageID) {
            return Bluebird.reject(`Page ${pageName} ID cannot be resolved`);
        }
        const query = wikidot_ajax_1.default(wikiUrl);
        const $ = await query({
            moduleName: 'pagerate/WhoRatedPageModule',
            pageId: pageID
        });
        const uids = Array.from($('span.printuser a:first-of-type')
            .map((i, a) => {
            const uid = $(a)
                .attr('onclick')
                .replace('WIKIDOT.page.listeners.userInfo(', '')
                .replace('); return false;', '');
            return parseInt(uid, 10);
        }));
        const votes = Array.from($('span:not([class])').map((i, span) => $(span).text().trim()));
        const userVotes = [];
        uids.forEach((uid, i) => {
            userVotes.push({ uid: Number(uid), vote: String(votes[i]) });
        });
        return userVotes;
    }
    async fetchPageRevisionsList(wikiUrl, pageName) {
        const pageUrl = `${wikiUrl}/${pageName}`;
        const pageID = await this.resolvePageId(pageUrl);
        if (!pageID) {
            return Bluebird.reject(`Page ${pageName} ID cannot be resolved`);
        }
        const query = wikidot_ajax_1.default(wikiUrl);
        const $ = await query({
            moduleName: 'history/PageRevisionListModule',
            page_id: pageID,
            options: { source: true },
            perpage: 3000
        });
        return Array.from($('.page-history tr:not(:first-child)').map((i, row) => {
            const revisionNumber = parseInt($(row).find('td:nth-child(1)').text(), 10);
            const rawUID = $(row)
                .find('.printuser a:first-child')
                .attr('onclick');
            const uid = rawUID
                ? parseInt(rawUID.replace('WIKIDOT.page.listeners.userInfo(', '').replace('); return false;', ''), 10)
                : -1;
            const date = $(row).find('td:nth-child(6)').text().replace('\n\t\t', '');
            const description = $(row).find('td:nth-child(7)').text();
            const revisionID = $(row)
                .find('a:first-child')
                .attr('onclick')
                .replace('showVersion(', '')
                .replace(')', '');
            return { number: revisionNumber, id: revisionID, uid, date, description };
        }));
    }
    async fetchPageRevisionContent(wikiUrl, revisionId) {
        const query = wikidot_ajax_1.default(wikiUrl);
        const $ = await query({
            moduleName: 'history/PageSourceModule',
            revision_id: revisionId
        });
        return querystring_1.unescape($('.page-source').text());
    }
    fetchThreadPage(wikiUrl, topicId, pageNumber) {
        const query = wikidot_ajax_1.default(wikiUrl);
        return query({
            moduleName: 'forum/ForumViewThreadPostsModule',
            t: topicId,
            pageNo: pageNumber
        });
    }
}
exports.wk = new WikidotKit(process.env.WIKIDOT_TOKEN || '');
exports.SITES = {
    'scp-ru': {
        name: 'scp-ru',
        url: 'http://scpfoundation.ru',
        title: 'SCP Foundation Russia',
        description: 'Russian branch main wiki',
        interwiki: true
    },
    'scp-wiki': {
        name: 'scp-wiki',
        url: 'http://www.scp-wiki.net',
        title: 'SCP Foundation',
        description: 'Original English wiki',
        interwiki: true
    },
    'wanderers-library': {
        name: 'wanderers-library',
        url: 'http://wanderers-library.wikidot.com',
        title: 'The Wanderers\' Library',
        description: 'Collaborative fiction exercise'
    },
    'fondationscp': {
        name: 'fondationscp',
        url: 'http://fondationscp.wikidot.com',
        title: 'Fondation SCP',
        description: 'The French Branch',
        interwiki: true
    },
    'scp-wiki-de': {
        name: 'scp-wiki-de',
        url: 'http://scp-wiki-de.wikidot.com',
        title: 'SCP Foundation in Deutschland',
        description: 'The German Branch',
        interwiki: true
    },
    'scp-pl': {
        name: 'scp-pl',
        url: 'http://scp-wiki.net.pl',
        title: 'Zabezpieczać Przechowywać Chronić',
        description: 'The Polish Branch',
        interwiki: true
    },
    'fondazionescp': {
        name: 'fondazionescp',
        url: 'http://fondazionescp.wikidot.com',
        title: 'Branca Italiana della SCP Foundation',
        description: 'The Italian branch',
        interwiki: true
    },
    'lafundacionscp': {
        name: 'lafundacionscp',
        url: 'http://lafundacionscp.wikidot.com',
        title: 'Rama Hispanohablante',
        description: 'The Spanish Branch',
        interwiki: true
    },
    'scp-wiki-cn': {
        name: 'scp-wiki-cn',
        url: 'http://scp-wiki-cn.wikidot.com',
        title: '控制，收容，保护',
        description: 'The Chinese Branch',
        interwiki: true
    },
    'scp-kr': {
        name: 'scp-kr',
        url: 'http://ko.scp-wiki.net',
        title: '확보, 격리, 보호',
        description: 'The Korean Branch',
        interwiki: true
    },
    'scp-jp': {
        name: 'scp-jp',
        url: 'http://ja.scp-wiki.net',
        title: '確保、収容、保護',
        description: 'The Japanese Branch',
        interwiki: true
    },
    'scp-th': {
        name: 'scp-th',
        url: 'http://scp-th.wikidot.com',
        title: 'จับ, เก็บ, เฝ้า',
        description: 'The Thai Branch',
        interwiki: true
    },
    'scp-ukrainian': {
        name: 'scp-ukrainian',
        url: 'http://scp-ukrainian.wikidot.com/',
        title: 'Охороняти. Утримувати. Захищати.',
        description: 'Ukrainian branch',
        interwiki: false
    }
};
exports.SITES_LIST = Object.keys(exports.SITES).reduce((acc, name) => acc.concat(exports.SITES[name]), []);
exports.SITES_DOMAINS = Object.keys(exports.SITES)
    .reduce((acc, name) => {
    return acc.concat(exports.SITES[name].url.replace('http://', ''));
}, []);
//# sourceMappingURL=wikidot-kit.js.map