"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const http = require("http");
const cookieParser = require("cookie-parser");
const bodyParser = require("body-parser");
const serveStatic = require("serve-static");
const routes_1 = require("../routes");
const sentry_1 = require("./sentry");
const pino_1 = require("./pino");
// const Arena: any = require('bull-arena');
const uuidv4 = require('uuid/v4');
const devMode = process.env.APP_ENV === 'development';
const httpLogger = require('pino-http')({
    logger: pino_1.default,
    genReqId: () => uuidv4(),
    serializers: {
        req: (req) => {
            if (devMode) {
                return `${req.method} ${req.url} [${req.id}]`;
            }
            else {
                return {
                    id: req.id,
                    method: req.method,
                    url: req.url,
                    headers: req.headers,
                    remoteAddress: req.connection && req.connection.remoteAddress,
                    remotePort: req.connection && req.connection.remotePort
                };
            }
        },
        res: (res) => {
            if (devMode) {
                const cleanedHeaders = String(res.header)
                    .replace(/(\r\n)+/g, ' • ')
                    .replace(/\s•\s$/, '');
                return `[${res.statusCode}] ${cleanedHeaders}`;
            }
            else {
                return {
                    statusCode: res.statusCode,
                    header: res.header
                };
            }
        }
    },
});
function generateServer() {
    const app = express();
    const router = express.Router(); // eslint-disable-line new-cap
    app.disable('x-powered-by');
    app.use(httpLogger);
    app.use(sentry_1.default.requestHandler());
    app.use(bodyParser.json());
    app.use(bodyParser.urlencoded({ extended: true }));
    app.use(cookieParser());
    if (devMode) {
        app.use(serveStatic(`${__dirname}/../../public`));
    }
    routes_1.default(router);
    // app.use('/', Arena({
    //     queues: [
    //         {
    //             name: 'common',
    //             hostId: 'scpdb.org',
    //             type: 'bee'
    //         }
    //     ]
    // }, {disableListen: true, basePath: '/o5/queue'}));
    app.use('/', router);
    app.use(sentry_1.default.errorHandler());
    app.use((err, req, res, next) => {
        if (req.log) {
            req.log.error(err, 'Unhandled error');
        }
        else {
            pino_1.default.error(err, 'Unhandled error');
        }
        res.end('Internal server error');
    });
    return function listen(port) {
        const server = http.createServer(app);
        server.on('listening', () => {
            pino_1.default.info(`Express server is listening on port ${server.address().port}`);
        });
        server.on('error', (error) => {
            if (error.syscall !== 'listen') {
                throw error;
            }
            switch (error.code) {
                case 'EACCES':
                    pino_1.default.error(`Port ${port} requires elevated privileges`);
                    process.exit(1);
                    break;
                case 'EADDRINUSE':
                    pino_1.default.error(`Port ${port} is already in use`);
                    process.exit(1);
                    break;
                default:
                    throw error;
            }
        });
        server.listen(port);
    };
}
exports.default = generateServer;
//# sourceMappingURL=express.js.map